% demo file for TCExt algorithm
clear;clc;close all;

for i = 1 : 10     % iteration number
    
t = 0:0.001:15;   % time
N = length(t);    % signal length

% generate three source signals with the same period length 20, but having different
% autocorreltion values at lag 20
S(1,:) = sin(2*pi*50*t);                                      % period: 20  sub-Gaussian
S(2,:) = sin(2*pi*50*t + pi/2) + 0.3 * randn(1,N);            % period: 20  sub-Gasusian
v = [1 : N];
S(3,:) = ((rem(v,20)-10)/9).^5 + 0.5 * randn(1,N);            % period: 20  near Gaussian

S(4,:) = rand(1,N)-0.5;                                       % sub-Gaussian noise
S(5,:) = ((rand(1,N)<.5)*2-1).*log(rand(1,N));                % super-Gaussian noise

S(6,:) = ((rem(v,36)-18)/17).^5 + 0.4 * randn(1,N);           % period: 34, near Gaussian
S(7,:) = ((rem(v,56)-28)/27).^10;                             % period: 56, super-Gaussian

% remove the mean of each source, and make the variance of each source to be unit
S = S - mean(S')' * ones (1,size(S, 2));
for nn = 1: size(S,1)
    standSig(nn,:) = S(nn,:)/std(S(nn,:));
end
S = standSig;

% display these sources (first 500 data points)
icashow(S(:,[1:500]));

% ACF = showAC(S,99,'title','\bfACF of Source Signals');   % show the autocorrelation function
%============================ Pre-set the Parameters for TCExt =============================
per = 56;
index = 1;   % extract the signal that has the third largest autocorrelation value at the lags
%============================================================================================

% generate mixing matrix
A = rand(size(S,1));

% generate the mixed signals
X = A * S;

% ==== Now we de-mix these signals =====

% whitening
[Z,V]  = whiten(X);   

% select suitable lags
delay = [1:5]*per;      

epsilon = 0.001;
maxIter = 300;
w = randn(size(X,1),1); w = w/norm(w);
threshold = 0.4;

% TCExt use 'type=2': i.e., based on autocorrelation and adaptive nonlinearities
mu = 0.5;
[y1,b1] = TCExt( Z, delay, 2, index,  mu, threshold, epsilon);

% standarizing, i.e., centering and normalizing
y1 = y1 - mean(y1);
y1 = y1/std(y1);


% Let TCExt use 'type=1': i.e., based on autocorrelation and standard kurtosis
mu = 1;
[y2,b2] = TCExt( Z, delay, 1, index,  mu, threshold, epsilon);
% standarizing, i.e., centering and normalizing
y2 = y2 - mean(y2);
y2 = y2/std(y2);


% eigBSE algorithm
[y3,b3] = eigBSE( Z, delay, index);   % run the algorithm
% standarizing, i.e., centering and normalizing
y3 = y3-mean(y3);
y3 = y3/std(y3);

% akExt algorithm (use the first period)
lambda = 150;
[y4,b4] = akExtr( Z, w, delay(1), lambda, epsilon, maxIter,1);figure;subplot(211);plot(y4(1:200));subplot(212);plot(S(6,1:200));
y4 = y4 - mean(y4);
y4 = y4/std(y4);


 
%================================== calculate performance ==========================
fprintf('\n\n');
p1= exPI( S(7,:), y1, 'dev');     fprintf('TCExt (type=2): %g dB;\n', p1 );
p2= exPI( S(7,:), y2, 'dev');     fprintf('TCExt (type=1): %g dB;\n', p2 ); 
p3= exPI( S(7,:), y3, 'dev');     fprintf('eigBSE        : %g dB; \n', p3 ); 
p4= exPI( S(7,:), y4, 'dev');     fprintf('akExt         : %g dB; \n', p4 ); 
                          

% record the performance in each iteration
P(i,:)=[p1 p2 p3 p4]

end
Y = [y1;y2;y3;y4];
icashow(Y(:,[1:500]),'title','extracted signals');




